#!/bin/bash

function show_usage ()
{
	echo $"Usage: $0 /path/to/kernel/source /path/to/module/source /path/to/build/dir kernel-version"
	exit 1
}

# If invoked as pre_build.sh --getsource <kernel-version> then the script attempts to copy the
# kernel sourcecode files needed for the build into the current directory
if [ $# = 2 -a "X$1" = "X--getsource" ]; then
	basekernel=`echo $2|sed 's/-.*//g'`
	set -- /usr/src/redhat/BUILD/kernel-$basekernel/linux-$basekernel `dirname $0` . "$2"
	unset basekernel
fi

[ $# != 4 ] && show_usage;

kernel_source_dir=$1
module_source_dir=$2
build_dir=$3
kernel_version=$4

VERSION_WO_SUFFIX=`echo ${kernel_version} | sed -e 's/smp\|BOOT\|bigmem\|hugemem\|enterprise\|debug//'`
BASE_KERNEL=2.6.5-1.358
# Files needed to build Advansys driver
ADVANSYS_FILE_LIST="advansys.c advansys.h hosts.h scsi.h scsi_module.c scsi_obsolete.h scsi_typedefs.h"
# Files needed to build scsi_mod module to get dependency right
SCSIMOD_FILE_LIST="constants.c hosts.c scsi.c scsicam.c scsi_devinfo.c scsi_error.c scsi_ioctl.c
	scsi_lib.c scsi_logging.h scsi_priv.h scsi_sysfs.c scsi_scan.c scsi_syms.c"

# In order to keep the size of the source tree down, yet still support multiple kernel
# versions, we start with the kernel source for the standard FC2 install kernel (2.6.5-1.358)
# and then apply zero or more patches to bring it up to date.
#
# None of this is necessary though if the kernel-sourcecode RPM is installed, as we just use
# the source from there if it's available.

# See if the kernel source appears to be present
if [ -f ${kernel_source_dir}/drivers/scsi/advansys.c -a \
     -f ${kernel_source_dir}/drivers/scsi/advansys.h ]; then
	# Yes; copy in files from the kernel source area
	for f in $ADVANSYS_FILE_LIST $SCSIMOD_FILE_LIST
	do
		if [ -f ${kernel_source_dir}/drivers/scsi/$f ]; then
			cp ${kernel_source_dir}/drivers/scsi/$f $build_dir
			echo "Using ${kernel_source_dir}/drivers/scsi/$f"
		else
			echo "$f not present in kernel source, skipping"
		fi
	done
elif [ -f /usr/src/linux-$VERSION_WO_SUFFIX/drivers/scsi/advansys.c -a \
       -f /usr/src/linux-$VERSION_WO_SUFFIX/drivers/scsi/advansys.h ]; then
	# Yes; copy in files from the kernel source area
	for f in $ADVANSYS_FILE_LIST $SCSIMOD_FILE_LIST
	do
		if [ -f /usr/src/linux-$VERSION_WO_SUFFIX/drivers/scsi/$f ]; then
			cp /usr/src/linux-$VERSION_WO_SUFFIX/drivers/scsi/$f $build_dir
			echo "Using /usr/src/linux-$VERSION_WO_SUFFIX/drivers/scsi/$f"
		else
			echo "$f not present in kernel source, skipping"
		fi
	done
else
	# No, use our own copy of the source
	for f in $ADVANSYS_FILE_LIST $SCSIMOD_FILE_LIST
	do
		cp $module_source_dir/kernel-source/$BASE_KERNEL/$f $build_dir
		echo "Using $module_source_dir/kernel-source/$BASE_KERNEL/$f"
	done
	# Create list of patches needed to bring source up to date
	PATCHLIST=`echo $VERSION_WO_SUFFIX |
		awk '	/^2\.6\.[0-5]-/		{ done = 1; exit }
			/^2\.6\.[0-9]+-/	{
						  # 2.6.6-
						  print "2.6.6-1.427-advansys.patch" }
			/^2\.6\.([7-9]|[1-9][0-9]+)-/ {
						  # 2.6.7-
						  print "2.6.7-1.494.2.2-advansys.patch" }
			/^2\.6\.([89]|[1-9][0-9]+)-/ {
						  # 2.6.8-* (except 2.6.8-1.521)
						  if ($1 == "2.6.8-1.521") { next }
						  print "2.6.8-1.541-advansys.patch" }
			/^2\.6\.(9|[1-9][0-9]+)-/ {
						  # 2.6.9-*
						  print "2.6.9-1.667-advansys.patch" }
			/^2\.6\.(9|[1-9][0-9]+)-/ {
						  # 2.6.9-* >= 1.700
						  if ($1 ~ "2.6.9-1.[0-6][0-9][0-9]") { next }
						  print "2.6.9-1.724-advansys.patch" }
			/^2\.6\.([1-9][0-9]+)-/ {
						  # 2.6.10-*
						  print "2.6.10-1.737-advansys.patch" }'`
	if [ -n "$PATCHLIST" ]; then
		cd $build_dir
		for p in $PATCHLIST
		do
			echo "Applying patch $p"
			patch -p1 --quiet < $module_source_dir/kernel-source/$p
		done
	fi
fi
